#include <amxmodx>
#include <sqlx>
#include <curl>
#include <json>

// remove it if you don't want to use the IP license before downloading new versions
// of your plugins (some servers returns different IP's due to host settings so the license won't work anyway.
#define USE_LICENSE

#if defined USE_LICENSE
#define DATABASE_HOST 		"localhost"
#define DATABASE_USER		"root"
#define DATABASE_PASS		""
#define DATABASE_NAME		"servers"
#endif

#define CURL_BUFFER_SIZE 512

// this will be where the json-formated file will be located on your website. The plugin will download it first.
new const g_szWebsitePluginsListFile[] = 	"https://www.yourwebsite.com/pluginsList.res"

// this will be where the downloaded file will be stored on the server.
new const g_szJsonFileDirectory[] = 		"resource/pluginsList.res"

// this is where your auto-download plugins will be located and downloaded afterwards.
new const g_szWebsitePluginsLink[] = 	"https://www.yourwebsite.com/plugins"

new bool:g_bServerNotLicensed

new Handle:g_sqlHandler

enum pFilesData
{
	pFileArchive[32],
	pFileDir[256],
	pArchiveSize
}

new g_iOldFileSize[1000][pFilesData]

new bool:g_bLogStarted

public plugin_init()
{
	register_plugin("Auto Plugins Updater", "1.1", "EFFEX")
	
	register_dictionary("autoupdater.txt")
}

public plugin_end()
{
	SQL_FreeHandle(g_sqlHandler)
}

#if defined USE_LICENSE
public plugin_cfg()
{	
	databaseConnection()
}

public databaseConnection()
{
	if(!SQL_SetAffinity("mysql"))
	{
		set_fail_state(fmt("Plugin Updater]: %L", LANG_SERVER, "MYSQL_INVALID_AFFINITY"))
		return
	}
	
	g_sqlHandler = SQL_MakeDbTuple(DATABASE_HOST, DATABASE_USER, DATABASE_PASS, DATABASE_NAME)
	
	new iErrorCode, szError[512], Handle:hSQLConnection
	hSQLConnection = SQL_Connect(g_sqlHandler, iErrorCode, szError, charsmax(szError))
	if(hSQLConnection != Empty_Handle)
	{
		SQL_FreeHandle(hSQLConnection)
	}
	else
	{
		set_fail_state("[Plugin Updater]: %L", LANG_SERVER, "MYSQL_COULDNT_CONNECT")
	}
	SQL_SetCharset(g_sqlHandler, "utf8")
	
	new szTableQuery[1024]
	formatex(szTableQuery, charsmax(szTableQuery), "CREATE TABLE IF NOT EXISTS `AutoUpdater_Authentification` (id int(11) PRIMARY KEY AUTO_INCREMENT, ip varchar(32) NOT NULL, on int(2));")
	SQL_ThreadQuery(g_sqlHandler, "SQL_QueryCreateTable", szTableQuery)
}

public SQL_QueryCreateTable(iFailState, Handle:hQuery, szError[], iError, szData[], iSize, Float:flQueueTime) 
{
	if(iFailState != TQUERY_CONNECT_FAILED)
	{
		new szServerIP[32]
		get_user_ip(0, szServerIP, charsmax(szServerIP))
		
		new szQuery[1024]
		formatex(szQuery, charsmax(szQuery), "SELECT ip, on FROM `AutoUpdater_Authentification` WHERE ip = ^"%s^";", szServerIP)
		SQL_ThreadQuery(g_sqlHandler, "SQL_CheckServerValidation", szQuery)
	}
}

public SQL_CheckServerValidation(iFailState, Handle:hQuery, szError[], iError, szData[], iSize, Float:flQueueTime) 
{
	if(SQL_NumResults(hQuery))
	{
		g_bServerNotLicensed = (SQL_ReadResult(hQuery, 1) == 1) ? false : true
	}
	else g_bServerNotLicensed = true
	
	if(file_exists(g_szJsonFileDirectory))
	{
		delete_file(g_szJsonFileDirectory)
	}
	set_task(2.0, "downloadJsonFile")
}
#else
public plugin_config()
{
	g_bServerNotLicensed = false
	if(file_exists(g_szJsonFileDirectory))
	{
		delete_file(g_szJsonFileDirectory)
	}
	set_task(2.0, "downloadJsonFile")
}
#endif

public downloadJsonFile()
{
	new data[1]
	data[0] = fopen(g_szJsonFileDirectory, "wb")
	
	new CURL:curl = curl_easy_init()
	curl_easy_setopt(curl, CURLOPT_BUFFERSIZE, CURL_BUFFER_SIZE)
	curl_easy_setopt(curl, CURLOPT_URL, g_szWebsitePluginsListFile)
	curl_easy_setopt(curl, CURLOPT_WRITEDATA, data[0])
	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, "writeJsonFile")
	curl_easy_perform(curl, "complite", data, sizeof(data))
}

public writeJsonFile(data[], size, nmemb, file)
{
	new actual_size = size * nmemb;
	fwrite_blocks(file, data, actual_size, BLOCK_CHAR)
	return actual_size
}

public complite(CURL:curl, CURLcode:code, data[])
{
	fclose(data[0])
	curl_easy_cleanup(curl)
	
	if(code != CURLE_WRITE_ERROR)
	{
		checkJsonFile()
	}
}

public checkJsonFile()
{
	new JSON:jConfigsFile = json_parse(g_szJsonFileDirectory, true, true)
	if(json_is_object(jConfigsFile))
	{
		new iFileSize = json_object_get_count(jConfigsFile), bool:bIsPlugin
		for(new i, iPluginId, iFileID, JSON:iObjectPos, szPluginName[32], szFileName[32], szFileDirectory[128];i < iFileSize;i++)
		{
			iObjectPos = json_object_get_value_at(jConfigsFile, i)
			if(json_object_get_bool(iObjectPos, "licensecheck") && g_bServerNotLicensed)
				continue
			
			json_object_get_string(iObjectPos, "filename", szFileName, charsmax(szFileName))
			
			bIsPlugin = bool:(containi(szFileName, ".amxx") != -1)
			if(bIsPlugin)
			{
				json_object_get_string(iObjectPos, "pluginname", szPluginName, charsmax(szPluginName))			
				if((iPluginId = is_plugin_loaded(szPluginName)) == -1)
					continue
			}
			json_object_get_string(iObjectPos, "directory", szFileDirectory, charsmax(szFileDirectory))
			
			if(bIsPlugin)
			{
				new szServerFileName[32], szServerFileVersion[22]
				get_plugin(iPluginId, szServerFileName, charsmax(szServerFileName), .version = szServerFileVersion, .len3 = charsmax(szServerFileVersion))
				if(!equal(szFileName, szServerFileName))
				{
					if(!g_bLogStarted)
					{
						log_to_file("autoupdater.log", "---------- Auto Plugins Updater Start ----------")
						g_bLogStarted = true
					}
					log_to_file("autoupdater.log", "%L", LANG_SERVER, "COULND_UPDATE_PLUGIN", szServerFileName, szFileName)
				}
				else
				{
					new szFileVersion[32]
					json_object_get_string(iObjectPos, "version", szFileVersion, charsmax(szFileVersion))
					if(!equal(szServerFileVersion, szFileVersion))
					{
						if(!g_bLogStarted)
						{
							log_to_file("autoupdater.log", "---------- Auto Plugins Updater Start ----------")
							g_bLogStarted = true
						}
						log_to_file("autoupdater.log", "%L", LANG_SERVER, "COULND_PLUGIN_UPDATED", szFileName, szFileVersion)
						webFileDownload(szFileDirectory, szFileName)
					}
				}
			}
			else 
			{				
				new szDir[256], bool:bLogDownload = json_object_get_bool(iObjectPos, "log")
				formatex(szDir, charsmax(szDir), "%s/%s", szFileDirectory, szFileName)
					
				if(file_exists(szDir))
				{					
					if(bLogDownload)
					{
						g_iOldFileSize[iFileID][pArchiveSize] = file_size(szDir)
					}
					delete_file(szDir)
				}

				if(bLogDownload)
				{
					copy(g_iOldFileSize[iFileID][pFileArchive], charsmax(g_iOldFileSize[][pFileArchive]), szFileName)
					copy(g_iOldFileSize[iFileID][pFileDir], charsmax(g_iOldFileSize[][pFileDir]), szDir)
					
					webFileDownload(szFileDirectory, szFileName, iFileID++)
				}
				else webFileDownload(szFileDirectory, szFileName)
			}
		}
		
		if(g_bLogStarted && bIsPlugin)
		{
			log_to_file("autoupdater.log", "---------- Auto Plugins Updater End ----------")
		}
	}
	json_free(jConfigsFile)
}

webFileDownload(szDirectory[], szFile[], iIndex = -1)
{
	new data[2]
	data[0] = fopen(fmt("%s/%s", szDirectory, szFile), "wb")
	data[1] = iIndex
	
	new CURL:curl = curl_easy_init()
	curl_easy_setopt(curl, CURLOPT_BUFFERSIZE, CURL_BUFFER_SIZE)
	curl_easy_setopt(curl, CURLOPT_URL, fmt("%s/%s", g_szWebsitePluginsLink, szFile))
	curl_easy_setopt(curl, CURLOPT_WRITEDATA, data[0])
	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, "downloadPlugin")
	curl_easy_perform(curl, "checkPluginDownload", data, sizeof(data))
}

public downloadPlugin(data[], size, nmemb, file)
{
	new actual_size = size * nmemb;
	fwrite_blocks(file, data, actual_size, BLOCK_CHAR)
	return actual_size
}

public checkPluginDownload(CURL:curl, CURLcode:code, data[])
{
	fclose(data[0])
	curl_easy_cleanup(curl)
	
	if(code != CURLE_WRITE_ERROR)
	{
		new iIndex = data[1]
		if(iIndex != -1)
		{
			if(g_iOldFileSize[iIndex][pArchiveSize])
			{
				set_task(1.0, "checkNewFileSize", iIndex)
			}
			else
			{
				if(!g_bLogStarted)
				{
					log_to_file("autoupdater.log", "---------- Auto Plugins Updater Start ----------")
					g_bLogStarted = true
				}
				
				log_to_file("autoupdater.log", "%L", LANG_SERVER, "ARCHIVE_DOWNLOADED", g_iOldFileSize[iIndex][pFileArchive])
				log_to_file("autoupdater.log", "---------- Auto Plugins Updater End ----------")
			}
		}
	}
}

public checkNewFileSize(iIndex)
{
	if(g_iOldFileSize[iIndex][pArchiveSize] != file_size(g_iOldFileSize[iIndex][pFileDir]))
	{
		log_to_file("autoupdater.log", "%L", LANG_SERVER, "ARCHIVE_DOWNLOADED", g_iOldFileSize[iIndex][pFileArchive])
		log_to_file("autoupdater.log", "---------- Auto Plugins Updater End ----------")
	}
}
